/*************************************************************************
Author:   Bjoern Biesenbach <bjoern@bjoern-b.de
Date:     28.09.2006	
Hardware: any AVR - tested on ATmega8 and 2 DS18B20

Using code from Martin Thomas, Peter Dannegger and maybe others.
*************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/eeprom.h>
#include <string.h>

#include "onewire.h"
#include "ds18x20.h"

#include "delay.h"
#include "main.h"
#include "lcd.h"

uint8_t clock_counter=0;	// Counter um eine genaue Sekunde zu erzeugen
uint8_t clock_counter_dcf=0;	// Counter um eine genaue Sekunde zu erzeugen
uint8_t seconds=0;		// Angezeigte Uhr: Sekunden
uint8_t minutes=0;		// Angezeigte Uhr: Minuten
uint8_t hours=0;		// Angezeigte Uhr: Stunden
uint8_t day=1;			// Angezeigte Uhr: Tage
uint8_t month=1;		// Angezeigte Uhr: Monat
uint8_t year=6;			// Angezeigte Uhr: Jahr
uint8_t weekday=0;		// Angezeigte Uhr: Wochentag

uint8_t Subzero[2];		// Positive oder negative Temperatur gemessen
uint16_t decicelsius[2];	// aktuelle Temperatur in 0.5°C Schritten


struct{
	unsigned char dcf:1;	// Status ob beim letzten IRQ DCF-Bit low oder 
				// high
	unsigned char sync:1;	// Interne Uhr mit DCF77 abgeglichen?
	unsigned char minute:1;	// Zum Vergleich ob empfangene Zeit korrekt
} stat;


uint8_t gSensorIDs[MAXSENSORS][OW_ROMCODE_SIZE];

uint8_t search_sensors(void)
{
	uint8_t i;
	uint8_t id[OW_ROMCODE_SIZE];
	uint8_t diff, nSensors;
	
	printf( "\r\nScanning Bus for DS18X20\r\n" );
	
	nSensors = 0;
	
	for( diff = OW_SEARCH_FIRST; 
		diff != OW_LAST_DEVICE && nSensors < MAXSENSORS ; )
	{
		DS18X20_find_sensor( &diff, &id[0] );
		
		if( diff == OW_PRESENCE_ERR ) {
			printf( "No Sensor found\r\n" );
			break;
		}
		
		if( diff == OW_DATA_ERR ) {
			printf( "Bus Error\r\n" );
			break;
		}
		
		for (i=0;i<OW_ROMCODE_SIZE;i++)
			gSensorIDs[nSensors][i]=id[i];
		
		nSensors++;
	}
	
	return nSensors;
}



void measure()
{
	uint16_t Decicelsius;
	uint8_t nSensor, i;
	uint8_t subzero, cel, cel_frac_bits;
	search_sensors();
	if ( DS18X20_start_meas( DS18X20_POWER_PARASITE, 
		&gSensorIDs[0][0] ) == DS18X20_OK ) {
		delay_ms(DS18B20_TCONV_12BIT);
		if ( DS18X20_read_meas( &gSensorIDs[0][0], &subzero,
				&cel, &cel_frac_bits) == DS18X20_OK ) {
			Decicelsius = DS18X20_temp_to_decicel(subzero, cel, cel_frac_bits);
			
		}
		if(Decicelsius!=850)
		{
			decicelsius[0]=Decicelsius;
			Subzero[0] = subzero;
		}
	}
	if ( DS18X20_start_meas( DS18X20_POWER_PARASITE, 
		&gSensorIDs[1][0] ) == DS18X20_OK ) {
		delay_ms(DS18B20_TCONV_12BIT);
		if ( DS18X20_read_meas( &gSensorIDs[1][0], &subzero,
				&cel, &cel_frac_bits) == DS18X20_OK ) {
			Decicelsius = DS18X20_temp_to_decicel(subzero, cel, cel_frac_bits);
		}
		if(Decicelsius!=850)
		{
			decicelsius[1]=Decicelsius;
			Subzero[1] = subzero;
		}
	}
}


SIGNAL(SIG_OUTPUT_COMPARE1A)
{
	TCNT1=0;
	dcf();
	clock();
	if(!stat.sync)
	{
		if(bit_is_set(PINC,PC0))
			PORTC &= ~(1<<PC1);
		else
			PORTC |= (1<<PC1);
	}
	else
		PORTC |= (1<<PC1);
}

int main(void)
{
	stat.dcf=1;
	stat.sync=0;
	stat.minute=1;
	uint8_t counter=0;
	DDRC &= ~(1<<PIN_DCF);
	DDRC |= (1<<PIN_LED);
	lcd_init();
	lcd_clear();
	TIMSK |= (1<<OCIE1A);
	TCCR1B |= (1<<CS10);
	OCR1A = 10000;
	sei();
	while(1)
	{
		measure();
	}
	return 0;
}
